import yfinance as yf
from datetime import date, datetime, timedelta
import pandas as pd


def str_to_date(trading_day: str):

    year = int(trading_day[:4])
    month = int(trading_day[5:7])
    day = int(trading_day[8:10])
    return date(year, month, day)


def get_trading_days(dir, start_date: date, end_date: date):

    df = pd.read_csv(f"{dir}/data_file/korea_trading_days.csv", index_col=0)
    df['날짜'] = df['날짜'].apply(lambda day: str_to_date(day))
    trading_days = df['날짜'].to_list()
    trading_days = [day for day in trading_days if start_date <= day <= end_date]
    return trading_days


def data_refactor(dir, start_date, end_date, df):

    df2 = {'날짜':[],'시가':[],'고가':[],'저가':[],'종가':[],'거래량':[]}

    now_date = start_date
    while now_date <= end_date:
        df2['날짜'].append(now_date)
        now_date = now_date + timedelta(days=1)

    now_date = start_date
    while now_date <= end_date :

        flag = 0

        try:
            value_exist_check = df['종가'].loc[now_date]
        except:
            flag = 1
            pass

        if flag == 0:
            df2['시가'].append(df['시가'].loc[now_date])
            df2['고가'].append(df['고가'].loc[now_date])
            df2['저가'].append(df['저가'].loc[now_date])
            df2['종가'].append(df['종가'].loc[now_date])
            df2['거래량'].append(df['거래량'].loc[now_date])

        else:
            df2['시가'].append(df2['시가'][-1])
            df2['고가'].append(df2['고가'][-1])
            df2['저가'].append(df2['저가'][-1])
            df2['종가'].append(df2['종가'][-1])
            df2['거래량'].append(df2['거래량'][-1])

        now_date = now_date + timedelta(days=1)

    df2 = pd.DataFrame(df2)
    df2['날짜'] = df2['날짜'].apply(lambda x : str_to_date(str(x)[:10]))

    trading_days = get_trading_days(dir, start_date, end_date)

    df3 = {'날짜':[],'시가':[],'고가':[],'저가':[],'종가':[],'거래량':[]}
    df2 = df2.set_index('날짜')

    for day in trading_days:

        df3['날짜'].append(day)
        df3['시가'].append(df2['시가'].loc[day])
        df3['고가'].append(df2['고가'].loc[day])
        df3['저가'].append(df2['저가'].loc[day])
        df3['종가'].append(df2['종가'].loc[day])
        df3['거래량'].append(df2['거래량'].loc[day])

    return df3


def generate_index_day_price(dir, day):

    # VIX 지수
    df = yf.download('^VIX')
    df = df.reset_index()

    df['날짜'] = df['Date']
    df['날짜'] = df['날짜'].apply(lambda x: str_to_date(str(x)[:10]))
    df['시가'] = round(df['Open'], 3)
    df['고가'] = round(df['High'], 3)
    df['저가'] = round(df['Low'], 3)
    df['종가'] = round(df['Close'], 3)
    df['거래량'] = round(df['Volume'], 3)

    df = df[['날짜', '시가', '고가', '저가', '종가', '거래량']]

    start_date = df['날짜'].iloc[0]
    end_date = df['날짜'].iloc[-1]
    df = df.set_index('날짜')

    df = data_refactor(dir, start_date, end_date, df)

    MSCI_Korea_ETF = pd.DataFrame(df)
    MSCI_Korea_ETF.to_csv(f"{dir}/data_file/VIX_지수.csv")
    print("VIX 지수 수집완료")

# MSCI 한국 ETF
    df = yf.download('EWY')
    df = df.reset_index()

    df['날짜'] = df['Date']
    df['날짜'] = df['날짜'].apply(lambda x: str_to_date(str(x)[:10]))
    df['시가'] = round(df['Open'], 3)
    df['고가'] = round(df['High'], 3)
    df['저가'] = round(df['Low'], 3)
    df['종가'] = round(df['Close'], 3)
    df['거래량'] = round(df['Volume'], 3)

    df = df[['날짜', '시가', '고가', '저가', '종가', '거래량']]

    start_date = df['날짜'].iloc[0]
    end_date = df['날짜'].iloc[-1]
    df = df.set_index('날짜')

    df = data_refactor(dir, start_date, end_date, df)

    MSCI_Korea_ETF = pd.DataFrame(df)
    MSCI_Korea_ETF.to_csv(f"{dir}/data_file/MSCI_한국_ETF.csv")
    print("MSCI 한국 ETF data 수집완료\n")


    trading_days = get_trading_days(dir, start_date, day)
    korea_end_day = trading_days[-1]
    print(f"한국거래일 : {korea_end_day}   미국거래일 : {str(end_date)[:10]}")



import json
import requests
import pandas as pd
import numpy as np
from datetime import date,timedelta
from time import sleep


def int_to_str(x: int) -> str:
    return "%06d" % x


def date_to_str(today):

    str_date = str(today)[:4] + str(today)[5:7] + str(today)[-2:]
    return str_date


def str_to_date(str_date):

    str_date = str_date.replace('-', '')
    year = int(str_date[:4])
    month = int(str_date[4:6])
    day = int(str_date[-2:])
    return date(year,month,day)


def get_trading_day(dir, start_date, end_date):

    trading_days = pd.read_csv(f"{dir}/data_file/korea_trading_days.csv")
    trading_days['날짜'] = trading_days['날짜'].apply(lambda x: str_to_date(x))
    trading_days = trading_days['날짜'].to_list()
    trading_days = [day for day in trading_days if start_date <= day <= end_date]
    return trading_days


def get_day_price(today, api):

    print(today)

    # 코스피 데이터 수집
    url = "https://data-dbg.krx.co.kr/svc/apis/sto/stk_bydd_trd"

    headers = {
        "Authorization": api,
        "Content-Type": "application/json"
    }

    payload = {
        "basDd": str(today),
        "isuCd": "005930"
    }

    j = json.loads(requests.post(url, headers=headers, json=payload).text)
    df = pd.json_normalize(j['OutBlock_1'])
    df = df.replace(',', '', regex=True)


    df = df.drop(['SECT_TP_NM', 'CMPPREVDD_PRC', 'FLUC_RT'], axis='columns')

    # 코스닥 데이터 수집
    url = "https://data-dbg.krx.co.kr/svc/apis/sto/ksq_bydd_trd"

    payload = {
        "basDd": str(today),
        "isuCd": "005930"
    }

    j = json.loads(requests.post(url, headers=headers, json=payload).text)
    df2 = pd.json_normalize(j['OutBlock_1'])
    df2 = df2.replace(',', '', regex=True)
    df2 = df2.drop(['SECT_TP_NM', 'CMPPREVDD_PRC', 'FLUC_RT'], axis='columns')

    # 코스피 코스닥 데이터 병합후 mapping
    df = pd.concat([df, df2])

    date_index = str_to_date(today)
    df['날짜'] = date_index

    cols_map = {'ISU_CD': '종목코드', 'ISU_NM': '종목명', 'MKT_NM': '마켓', '날짜': '날짜',
                'TDD_CLSPRC': '종가', 'TDD_OPNPRC': '시가', 'TDD_HGPRC': '고가', 'TDD_LWPRC': '저가',
                'ACC_TRDVOL': '거래량', 'ACC_TRDVAL': '거래대금', 'MKTCAP': '시가총액', 'LIST_SHRS': '상장주식수'
                }

    df = df.rename(columns=cols_map)
    df = df[['종목코드', '종목명', '날짜', '마켓', '종가', '시가', '고가', '저가', '거래량', '거래대금', '시가총액', '상장주식수']]

    return df


def generate_day_price(dir, day, api):

    bef_day_price = pd.read_csv(f"{dir}/data_file/전체데이터.csv", index_col=0)

    try:
        bef_day_price['날짜'] = bef_day_price['날짜'].apply(lambda day: str_to_date(day))
        bef_day_price['종목코드'] = bef_day_price['종목코드'].apply(lambda x: int_to_str(x))
    except:
        pass

    start_date = bef_day_price['날짜'].iloc[-1]
    start_date = start_date + timedelta(days=1)

    end_date = day

    if start_date > end_date:
        print("수집할 가격데이터가 없습니다")

    trading_days = get_trading_day(dir, start_date, end_date)


    if len(trading_days) == 0:

        print("수집할 가격데이터가 없습니다")

    else:

        trading_days = [date_to_str(day) for day in trading_days]
        aft_day_price = get_day_price(trading_days[0], api)

        for i,now_day in enumerate(trading_days):

            if i == 0:
                continue

            df2 = get_day_price(now_day, api)
            aft_day_price = pd.concat([aft_day_price,df2])
            sleep(1)


        try:
            day_price = pd.concat([bef_day_price, aft_day_price])
            day_price = day_price.reset_index()
        except:
            pass
        # ,level_0,index,종목코드,종목명,날짜,마켓,종가,시가,고가,저가,거래량,거래대금,시가총액,상장주식수,금융투자순매수,금융투자순거래대금,보험순매수,보험순거래대금,투신순매수,투신순거래대금,은행순매수,은행순거래대금,기타금융순매수,기타금융순거래대금,연기금순매수,연기금순거래대금,기관순매수,기관순거래대금,기타법인순매수,기타법인순거래대금,개인순매수,개인순거래대금,외국인순매수,외국인순거래대금,기타외국인순매수,기타외국인순거래대금

        day_price = day_price[['종목코드', '종목명', '날짜', '마켓', '종가', '시가', '고가', '저가', '거래량', '거래대금', '시가총액', '상장주식수']]

        day_price.to_csv(f"{dir}/data_file/전체데이터.csv")


from datetime import date, timedelta
import pandas as pd
from pandas import DataFrame
import math

from buke_pypi.indicator import sma, ibs, pivot_standard, stochastic_fast_k, stochastic_fast_d, \
    pivot_standard_for_sellprice, 순거래대금
from buke_pypi.function import rank
from buke_pypi.parameter import Market, MovingAverage
from buke_pypi.tools import get_stock_exchange_tax, get_ask_price, get_bid_price


def what_day_is_it(date):

    days = ['월요일', '화요일', '수요일', '목요일', '금요일', '토요일', '일요일']
    day = date.weekday()
    return days[day]


def str_to_date(trading_day: str) -> date:
    return date(int(trading_day[:4]), int(trading_day[5:7]), int(trading_day[8:]))


def int_to_str(x: int) -> str:

    try:
        x = int(x)
        return "%06d" % x
    except:
        return x


def read_trading_days(dir, start_date: date, end_date: date):

    df = pd.read_csv(f"{dir}/data_file/korea_trading_days.csv", index_col=0)
    df['날짜'] = df['날짜'].apply(lambda day: str_to_date(day))
    trading_days = df['날짜'].to_list()
    trading_days = [day for day in trading_days if start_date <= day <= end_date]
    return trading_days


def read_day_price(dir, start_date: date, end_date: date):
    df = pd.read_csv(f"{dir}/data_file/수급데이터_2022_to_{end_date.year}.csv", index_col=0)
    df['날짜'] = df['날짜'].apply(lambda day: str_to_date(day))
    # try:
    df['종목코드'] = df['종목코드'].apply(lambda x: int_to_str(x))
    # except:
    #     pass
    return df


def read_index_day_price(dir, start_date: date, end_date: date):

    df = pd.read_csv(f"{dir}/data_file/MSCI_한국_ETF.csv", index_col=0)
    df['날짜'] = df['날짜'].apply(lambda day: str_to_date(day))
    return df


def read_index_history(dir, start_date: date, end_date: date):

    ks_history = pd.read_csv(f"{dir}/data_file/코스피_종목리스트_2022_to_{end_date.year}.csv", index_col=0)
    ks_history['날짜'] = ks_history['날짜'].apply(lambda day: str_to_date(day))
    ks_history = ks_history.set_index('날짜')

    kq_history = pd.read_csv(f"{dir}/data_file/코스닥_종목리스트_2022_to_{end_date.year}.csv")
    kq_history['날짜'] = kq_history['날짜'].apply(lambda day: str_to_date(day))
    kq_history = kq_history.set_index('날짜')
    return ks_history, kq_history


def add_indicator(day_price: DataFrame, index_day_price: DataFrame) -> DataFrame:

    group_list = []
    grouped = day_price.groupby('종목코드')
    for key, group in grouped:
        group = group.reset_index(drop=True)
        group['거래대금_이평선_5'] = sma(group['거래대금'], 5)
        group['pivot_기준선'] = pivot_standard(group['고가'], group['저가'], group['종가'])
        group['pivot_기준선_for_sell'] = pivot_standard_for_sellprice(group['고가'], group['저가'], group['종가'])

        group['순거래대금'] = 순거래대금(group['종가'], group['거래량'], 1)
        group['순거래대금_이평선_5'] = sma(group['순거래대금'], 5)

        group['종가_이평선_3'] = sma(group['종가'], 3)
        group['종가_이평선_5'] = sma(group['종가'], 5)
        group['종가_이평선_20'] = sma(group['종가'], 20)
        group['거래대금_이평선_10'] = sma(group['거래대금'], 10)

        group['스토캐스틱_fast'] = stochastic_fast_k(group['고가'], group['저가'], group['종가'], 20)
        group['스토캐스틱_slow'] = stochastic_fast_d(group['고가'], group['저가'], group['종가'], 20,
                                                     5, MovingAverage.sma)
        group['ibs'] = ibs(group['고가'], group['저가'], group['종가'])

        group_list.append(group)
    day_price = pd.concat(group_list, axis=0)
    day_price = day_price.reset_index(drop=True)

    group_list = []
    grouped = day_price.groupby('날짜')
    for key, group in grouped:
        group = group.reset_index(drop=True)
        group['거래대금_이평선_5_순위'] = rank(group['거래대금_이평선_5'])
        group['순거래대금_이평선_5_순위'] = rank(group['순거래대금_이평선_5'])
        group['시가총액_순위'] = rank(group['시가총액'])

        group_list.append(group)
    day_price = pd.concat(group_list, axis=0)
    day_price = day_price.reset_index(drop=True)

    day_price_dict = {}
    grouped = day_price.groupby('종목코드')
    for key, group in grouped:

        increase_condition1 = group['종가_이평선_5'] > group['종가_이평선_5'].shift(5)
        increase_condition2 = group['거래대금_이평선_10'] > group['거래대금_이평선_10'].shift(10)
        increase_condition3 = group['종가'] / group['종가_이평선_3'] < 1
        increase_condition4 = group['종가'] / group['종가_이평선_20'] > 1.05
        increase_condition5 = group['ibs'] < 0.7

        increase_condition = increase_condition1 & increase_condition2 & increase_condition3 & increase_condition4 & \
                             increase_condition5

        liquidity_condition1 = group['거래대금_이평선_5_순위'] > 0.5
        liquidity_condition2 = group['시가총액_순위'] > 0.3
        liquidity_condition3 = group['순거래대금_이평선_5_순위'] > 0.5

        liquidity_condition = liquidity_condition1 & liquidity_condition2 & liquidity_condition3

        # Result
        group['#final_result'] = increase_condition & liquidity_condition

        group['#priority'] = group['스토캐스틱_slow'] - group['스토캐스틱_fast']

        group = group.set_index('날짜')
        day_price_dict[key] = group

    # Market Timing
    index_day_price['이평선_3'] = sma(index_day_price['종가'], 3)
    index_day_price['이평선_5'] = sma(index_day_price['종가'], 5)
    index_day_price['이평선_10'] = sma(index_day_price['종가'], 10)
    # index_day_price['pct_change_21'] = pct_change(index_day_price['close'], 21)
    index_day_price['#market_timing'] = (index_day_price['종가'] > index_day_price['이평선_3']) | \
                                        (index_day_price['종가'] > index_day_price['이평선_5']) | \
                                        (index_day_price['종가'] > index_day_price['이평선_10'])
    index_day_price = index_day_price.set_index('날짜')

    return day_price_dict, index_day_price



def back_testing(dir, start_date, end_date):

    # 0. 변수
    max_basket_size = 10
    liquidation_holding_days = 1
    cut_period = 0
    seed_money = 10000000
    fee = 0.00015  # when using kiwoom
    basket = {}  # key:ticker, value:buy_price, stock_num, holding_days
    pnl = {'date': [start_date], 'balance': [seed_money]}
    p_len = []


    # 1. 거래일 가져오기
    trading_days_list = read_trading_days(dir, start_date, end_date)

    # 2. 일봉 가져오기
    day_price = read_day_price(dir, start_date, end_date)

    index_day_price = read_index_day_price(dir, start_date, end_date)

    # 3. 지수 종목 구성 내역 가져오기
    kospi_history, kosdaq_history = read_index_history(dir, start_date, end_date)

    # 4. 매수 조건 및 우선순위 생성
    day_price, index_day_price = add_indicator(day_price, index_day_price)
    print(f"Finished data preprocessing...")
    # print(day_price[['#final_result','#priority','market_timing']])
    # print(day_price.tail())


    # 5. 백테스트
    for i, day in enumerate(trading_days_list):

        # if i > 1:
        # 당일 코스피, 코스닥 구성 종목 내역 가져오기
        kospi_ticker_list = kospi_history['종목리스트'].loc[day].split(',')
        kosdaq_ticker_list = kosdaq_history['종목리스트'].loc[day].split(',')

        # Market Timing
        buy_check = 1
        try:
            yesterday_market_time = bool(index_day_price['#market_timing'].loc[trading_days_list[i - 1]])
        except:
            buy_check = 0

            # 매수
        if len(basket) < max_basket_size and buy_check == 1:
            # buy at open price
            pre_buy_list = []
            for ticker in kospi_ticker_list + kosdaq_ticker_list:

                if ticker in basket:
                    continue
                try:
                    yesterday_buy_condition = bool(
                        day_price[ticker]['#final_result'].loc[trading_days_list[i - 1]])

                    # Market Time

                    if yesterday_market_time and yesterday_buy_condition :
                        priority_value = float(day_price[ticker]['#priority'].loc[trading_days_list[i - 1]])
                        ticker_data = (priority_value, ticker)
                        pre_buy_list.append(ticker_data)
                except:
                    pass

            if len(pre_buy_list) > 0:
                p_len.append(len(pre_buy_list))
                pre_buy_list.sort(reverse=True)
                # print(f"{day}, {pre_buy_list}")
                for priority_value, ticker in pre_buy_list[:max_basket_size - len(basket)]:
                    yesterday_low = int(day_price[ticker]['저가'].loc[trading_days_list[i - 1]])
                    tomorrow_pivot_standard_price = float(day_price[ticker]['pivot_기준선_for_sell'].loc[day])

                    # yesterday_close = int(day_price[ticker]['close'].loc[trading_days_list[i - 1]])

                    if ticker in kospi_ticker_list:
                        today_target_buy_price = get_ask_price(yesterday_low * 0.98, Market.kospi)
                        sell_price = get_bid_price(tomorrow_pivot_standard_price * 1.01,
                                                          Market.kospi)
                    else:
                        today_target_buy_price = get_ask_price(yesterday_low * 0.98, Market.kosdaq)
                        sell_price = get_bid_price(tomorrow_pivot_standard_price * 1.01,
                                                   Market.kosdaq)

                    today_open = int(day_price[ticker]['시가'].loc[day])
                    today_low = int(day_price[ticker]['저가'].loc[day])
                    today_name = day_price[ticker]['종목명'].loc[day]

                    real_buy_price = 0.

                    if today_open <= today_target_buy_price:
                        buy_price = math.ceil(today_open * (1 + fee))
                        real_buy_price = today_open
                    elif today_low < today_target_buy_price:
                        buy_price = math.ceil(today_target_buy_price * (1 + fee))
                        real_buy_price = today_target_buy_price
                    else:
                        continue

                    stock_num = (pnl['balance'][-1] // max_basket_size) // buy_price
                    stock_info = {'stock_num': stock_num, 'buy_price': buy_price, 'holding_days': 0, 'name':today_name, 'buy_date':day, 'real_buy_price':real_buy_price, 'sell_price':sell_price}

                    basket[ticker] = stock_info
                    #print(f"<Buy> date: {day}, ticker: {ticker}, buy_price: {buy_price}")

        # 매도
        today_profit = 0
        if len(basket) > 0:
            tax = get_stock_exchange_tax(day)

            sell_list = []
            for ticker in basket:

                holding_condition1 = basket[ticker]['holding_days'] >= liquidation_holding_days

                if holding_condition1:

                    yesterday_close = int(day_price[ticker]['종가'].loc[trading_days_list[i - 1]])
                    today_open = int(day_price[ticker]['시가'].loc[day])
                    today_high = int(day_price[ticker]['고가'].loc[day])
                    today_close = int(day_price[ticker]['종가'].loc[day])
                    # tomorrow_high = int(day_price[ticker]['고가'].loc[trading_days_list[i + 1]])
                    # tomorrow_low = int(day_price[ticker]['저가'].loc[trading_days_list[i + 1]])
                    pivot_standard_price = float(day_price[ticker]['pivot_기준선'].loc[day])
                    tomorrow_pivot_standard_price = float(day_price[ticker]['pivot_기준선_for_sell'].loc[day])

                    buy_date = basket[ticker]['buy_date']
                    sell_date = day
                    name = basket[ticker]['name']

                    # 익일 상폐 여부 확인
                    # trash_condition = ticker not in tomorrow_kospi_ticker_list + tomorrow_kosdaq_ticker_list

                    if yesterday_market_time is True:

                        if ticker in kospi_ticker_list:
                            target_sell_price = get_bid_price(pivot_standard_price * 1.01,
                                                              Market.kospi)
                            tomorrow_target_sell_price = get_bid_price(tomorrow_pivot_standard_price * 1.01,
                                                              Market.kospi)
                        else:
                            target_sell_price = get_bid_price(pivot_standard_price * 1.01,
                                                              Market.kosdaq)
                            tomorrow_target_sell_price = get_bid_price(tomorrow_pivot_standard_price * 1.01,
                                                              Market.kosdaq)

                        today_vol = int(day_price[ticker]['거래량'].loc[day])
                        basket[ticker]['sell_price'] = tomorrow_target_sell_price

                        if today_open >= target_sell_price:
                            sell_price = today_open
                        elif today_high > target_sell_price:
                            sell_price = target_sell_price
                        else:
                            basket[ticker]['holding_days'] += 1
                            continue
                    else:

                        if ticker in kospi_ticker_list:
                            target_sell_price = get_bid_price(pivot_standard_price * 1.01,
                                                              Market.kospi)
                            tomorrow_target_sell_price = get_bid_price(tomorrow_pivot_standard_price * 1.01,
                                                                       Market.kospi)
                        else:
                            target_sell_price = get_bid_price(pivot_standard_price * 1.01,
                                                              Market.kosdaq)
                            tomorrow_target_sell_price = get_bid_price(tomorrow_pivot_standard_price * 1.01,
                                                                       Market.kosdaq)

                        today_vol = int(day_price[ticker]['거래량'].loc[day])
                        # tomorrow_vol = int(day_price[ticker]['거래량'].loc[trading_days_list[i + 1]])
                        basket[ticker]['sell_price'] = tomorrow_target_sell_price

                        if today_open >= target_sell_price:
                            sell_price = today_open
                        elif today_high > target_sell_price:
                            sell_price = target_sell_price
                        else:
                            basket[ticker]['holding_days'] += 1
                            continue

                    profit = basket[ticker]['stock_num'] * (
                            math.floor(sell_price * (1 - (tax + fee))) - basket[ticker]['buy_price'])
                    today_profit += profit


                    #print(f"<Sell> date: {day}, ticker: {ticker}, sell_price: {sell_price}, profit: {profit}")
                    sell_list.append(ticker)

                else:
                    basket[ticker]['holding_days'] += 1
            for ticker in sell_list:
                basket.pop(ticker)


        # Balance Update
        pnl['date'].append(day)
        pnl['balance'].append(pnl['balance'][-1] + today_profit)

    return basket


def Buke_S002_sell_list(dir, today):


    path = f"{dir}/trade_list/{today}"

    end_date = today# today는 토요일 or 일요일
    start_date = end_date - timedelta(days=30)

    print(start_date, end_date)
    trading_days_list = read_trading_days(dir, start_date, end_date)

    real_start_date = trading_days_list[0]
    real_end_date = trading_days_list[-1]
    print("start_date : ", what_day_is_it(real_start_date))
    print("end_date   : ", what_day_is_it(real_end_date))
    print()

    basket = back_testing(dir, real_start_date, real_end_date)


    file_name = "sell_2"
    sell_basket = {'종목코드': [], '종목명': [], '매도가격': []}

    for ticker in basket:

        sell_price = basket[ticker]['sell_price']
        name = basket[ticker]['name']

        sell_basket['종목코드'].append(ticker)
        sell_basket['종목명'].append(name)
        sell_basket['매도가격'].append(sell_price)

    sell_basket = pd.DataFrame(sell_basket)
    sell_basket.to_csv(f"{path}/{file_name}_{str(today)[-5:]}.txt")


import json
import requests
import pandas as pd
import numpy as np
from datetime import date,timedelta
from time import sleep


def str_to_6str(x) -> str:

    if type(x) == int:
        return "%06d" % x

    else:
        str_len = len(x)
        zero_cnt = 6 - str_len

        for i in range(0, zero_cnt):
            x = '0' + x

        return x


def date_to_str(today):

    str_date = str(today)[:4] + str(today)[5:7] + str(today)[-2:]
    return str_date


def str_to_date(str_date):

    str_date = str_date.replace('-', '')
    year = int(str_date[:4])
    month = int(str_date[4:6])
    day = int(str_date[-2:])
    return date(year,month,day)


def get_day_price(dir, end_date):

    day_price = pd.read_csv(f"{dir}/data_file/전체데이터.csv", index_col=0)

    return day_price



def generate_stock_list(dir, day):

    day_price = get_day_price(dir, day)

    kospi_date_groupby_df = {'날짜':[], '종목리스트':[]}
    kosdaq_date_groupby_df = {'날짜':[], '종목리스트':[]}
    date_group = day_price.groupby('날짜')

    for date, group in date_group:

        kospi_daily = []
        kosdaq_daily = []

        group['종목코드'] = group['종목코드'].apply(lambda x : str_to_6str(x))

        for i in range(len(group['마켓'])):

            if group['마켓'].iloc[i] == 'KOSPI':
                kospi_daily.append(group['종목코드'].iloc[i])
            else:
                kosdaq_daily.append(group['종목코드'].iloc[i])

        kospi_daily = str(kospi_daily)[1:-1]
        kospi_daily = kospi_daily.replace('\'','').replace(' ','')
        kospi_date_groupby_df['날짜'].append(date)
        kospi_date_groupby_df['종목리스트'].append(kospi_daily)

        kosdaq_daily = str(kosdaq_daily)[1:-1]
        kosdaq_daily = kosdaq_daily.replace('\'', '').replace(' ', '')
        kosdaq_date_groupby_df['날짜'].append(date)
        kosdaq_date_groupby_df['종목리스트'].append(kosdaq_daily)

    df = pd.DataFrame(kospi_date_groupby_df)
    df.to_csv(f"{dir}/data_file/코스피_종목리스트.csv")

    df = pd.DataFrame(kosdaq_date_groupby_df)
    df.to_csv(f"{dir}/data_file/코스닥_종목리스트.csv")



from enum import Enum


class Market(Enum):
    kospi = 1
    kosdaq = 2
    etf = 3


class Universe(Enum):
    total = 1
    top350 = 2
    kospi = 3
    kospi200 = 4
    kosdaq = 5
    kosdaq150 = 6


class ExchangeRate(Enum):
    dollar = "USDKRW"
    euro = "EURKRW"
    yen = "JPYKRW"


class MovingAverage(Enum):
    sma = 1
    ema = 2
    ewma = 3
    wma = 4


class KindOfAverage(Enum):
    average = 1
    min = 2
    max = 3
    first = 4
    dense = 5


class UnitPeriod(Enum):
    month = 1
    year = 2



from datetime import date, timedelta
import pandas as pd
import json
import requests
import os
import numpy as np


def date_to_str(today):

    str_date = str(today)[:4] + str(today)[5:7] + str(today)[-2:]
    return str_date


def str_to_date(trading_day: str):
    year = int(trading_day[:4])
    month = int(trading_day[5:7])
    day = int(trading_day[-2:])
    return date(year, month, day)


def get_trading_days(dir):
    df = pd.read_csv(f"{dir}/data_file/korea_trading_days.csv", index_col=0)
    df['날짜'] = df['날짜'].apply(lambda day: str_to_date(day))
    return df


def get_day_price(code: str, start_date:str):
    """
    네이버 금융 일별 시세 최대 수집

    :param code: 종목 코드 (예: 005930)
    :param sleep: 페이지 간 대기 시간 (차단 방지)
    :param as_json: True면 JSON 반환
    """

    sleep: float = 0.5

    url = "https://finance.naver.com/item/sise_day.naver"
    headers = {"User-Agent": "Mozilla/5.0"}

    page = 1
    all_data = []
    last_date = None

    try:

        while True:
            params = {"code": code, "page": page}
            res = requests.get(url, params=params, headers=headers)
            res.encoding = "euc-kr"

            soup = BeautifulSoup(res.text, "html.parser")
            table = soup.find("table", class_="type2")

            if not table:
                break

            rows = table.find_all("tr")
            page_data = []

            for row in rows:
                cols = row.find_all("td")
                if len(cols) != 7:
                    continue

                date = cols[0].get_text(strip=True)
                date = date.replace(".", "")

                # 기존에 없는 데이터만 수집
                if date < start_date:
                    break
                # 더 이상 과거 데이터가 없을 때
                if date == last_date:
                    continue

                page_data.append({
                    "날짜": date,
                    # "close": int(cols[1].get_text(strip=True).replace(",", "")),
                    # "diff": cols[2].get_text(strip=True),
                    # "open": int(cols[3].get_text(strip=True).replace(",", "")),
                    # "high": int(cols[4].get_text(strip=True).replace(",", "")),
                    # "low": int(cols[5].get_text(strip=True).replace(",", "")),
                    # "volume": int(cols[6].get_text(strip=True).replace(",", ""))
                })

            if not page_data:
                break  # 더 이상 데이터 없음 → 종료

            last_date = page_data[-1]["날짜"]
            all_data.extend(page_data)

            # print(f"[INFO] page {page} 수집 완료 ({len(page_data)} rows)")
            page += 1
            time.sleep(sleep)

        df = pd.DataFrame(all_data)
        df = df.drop_duplicates(subset="날짜").sort_values("날짜")

        df['날짜'] = df['날짜'].apply(lambda x : datetime.strptime(x, "%Y%m%d"))
        df['날짜'] = df['날짜'].apply(lambda x : x.strftime("%Y-%m-%d"))
        df['날짜'] = df['날짜'].apply(lambda x: str_to_date(x))

    except:
        print("(EXCEPT)[get_day_price] 수집할 거래일이 없습니다.")
        return

    return df


def generate_trading_days(dir, day):

    try:
        os.makedirs(f"{dir}/data_file")
    except OSError:
        if not os.path.isdir(f"{dir}/data_file"):
            raise

    bef_trading_days = get_trading_days(dir)
    start_date = bef_trading_days['날짜'].iloc[-1]
    start_date = start_date + timedelta(days=1)

    end_date = day

    if start_date > end_date :

        print("수집할 거래일이 없습니다")

    else :

        str_start_date = date_to_str(start_date)
        str_end_date = date_to_str(end_date)

        try:

            str_start_date = date_to_str(start_date)

            new_trading_days = get_day_price("005930", str_start_date)

            df = pd.concat([bef_trading_days, new_trading_days])
            df = df.sort_values("날짜")
            df.index = np.arange(len(df))

            df.to_csv(f"{dir}/data_file/korea_trading_days.csv")

        except:

            print("(EXCEPT) 수집할 거래일이 없습니다.")





from datetime import datetime, date
import numpy as np
import pandas as pd
from pandas import DataFrame
import math


def str_to_date(trading_day: str) -> date:
    return date(int(trading_day[:4]), int(trading_day[5:7]), int(trading_day[8:]))


def int_to_str(x: int) -> str:
    return "%06d" % x


def read_day_price(dir, end_date: date):

    df = pd.read_csv(f"{dir}/data_file/전체데이터_2023_to_{end_date.year}.csv", index_col=0)
    df['날짜'] = df['날짜'].apply(lambda day: str_to_date(day))
    # df['종목코드'] = df['종목코드'].apply(lambda x: int_to_str(x))
    return df


def add_indicator(dir, day_price: DataFrame) -> DataFrame:

    group_list = []
    기관_sum = {'날짜':[], '종가':[]}
    외인_sum = {'날짜':[], '종가':[]}

    grouped = day_price.groupby('날짜')
    for key, group in grouped:
        group = group.reset_index(drop=True)

        기관_ts_sum = sum(group['기관순거래대금'])
        기관_sum['날짜'].append(key)
        기관_sum['종가'].append(기관_ts_sum)

        외인_ts_sum = sum(group['외국인순거래대금'])
        외인_sum['날짜'].append(key)
        외인_sum['종가'].append(외인_ts_sum)

        group_list.append(group)

    기관_sum = pd.DataFrame(기관_sum)
    기관_sum.to_csv(f"{dir}/data_file/기관순거래대금합.csv")

    외인_sum = pd.DataFrame(외인_sum)
    외인_sum.to_csv(f"{dir}/data_file/외인순거래대금합.csv")

def generate_trval_data(dir, day):


    # 1. 일봉 가져오기
    day_price = read_day_price(dir, day)

    # 2. 매수 조건 및 우선순위 생성
    add_indicator(dir, day_price)


from setuptools import setup, find_packages

setup(
    name="buke_pypi",
    version="3.1.1",
    packages=find_packages(),
    python_requires=">=3.8",
)

# python setup.py sdist bdist_wheel
# twine upload dist/*
